//
//  FjRecogEngine.h
//  FjRecogEngine
//
//  Created by FNST on 12-6-18.
//  Copyright (c) 2012 __FUJITSU_LIMITED__. All rights reserved.
//

#import <Foundation/Foundation.h>

@class RecoChar;
@class OnlinePattern;


#define _USE_EXTERNAL_FILTER

//エラータイプの一覧
/**
 * エラーなし。
 */
const static int NO_ERROR = 0x0000;
/**
 * エンジンは初期化されていません。
 */
const static int ENGINE_UNINITIALIZED_ERROR = 0x0001;
/**
 * メモリ確保に失敗。
 */
const static int ENGINE_ALLOC_MEMORY_ERROR = 0x0002;
/**
 * ファイルの文字コードは正しくありません。
 */
const static int ENGINE_FILE_ENCODING_ERROR = 0x0003;
/**
 * 最大種類値が規定範囲外です。
 */
const static int MAX_CATEGORY_OUT_OF_RANGE_ERROR = 0x0010;
/**
 * フィルタリングルールを記載する文の形式が正しくありません（外部ファイルで認識範囲を設定する場合のみ）。
 */
const static int FILTER_RULE_FORMAT_ERROR = 0x0011;
/**
 * フィルタリングルール設定時の種類値が規定範囲外です。
 */
const static int FILTER_CATEGORY_OUT_OF_RANGE_ERROR = 0x0012;
/**
 * 文字コードが規定範囲外です。
 */
const static int ENCODE_OUT_OF_RANGE_ERROR = 0x0013;
/**
 * 文字の種類値が規定範囲外です。
 */
const static int SETTING_CATEGORY_OUT_OF_RANGE_ERROR = 0x0014;
/**
 * 文字種類を設定する文の形式が正しくありません（外部ファイルで認識範囲を設定する場合のみ）。
 */
const static int SEETING_RULE_FORMAT_ERROR = 0x0015;
/**
 * パラメーターは正しくありません。
 */
const static int PARAMETER_ERROR = 0x0016;
//-----------------------------------------//
//the max predicting string number
const static int MAX_PREDICT_NUM = 10;
const static int MAX_CHAR_COUNT = 100;


@interface FjRecogEngine : NSObject{
    
    //the error type of last function invoke
    int m_lastError;
    //the max category
    int m_maxCategory;
    //the max number of recognition result engine can save;
    int m_maxTextLen;
    //the max number of characters for each recognition result;
    int m_maxCandCnt;
    //the height of the recognition area;
    int m_height;
    //the width of the recognition area;
    int m_width;
    //the recognition instance;
    RecoChar* m_reco;
    //used to save the recognition result; //RecogResult []
    NSMutableArray* m_lattice;
    //used to save the points;
    OnlinePattern* m_pattern;
    //used to save the points of all chars
    NSMutableArray* m_patterns; // OnlinePattern []
    //if the recognition result engine saved is changed;
    BOOL m_replaceFlag;
    //if the engine is initialized
    BOOL m_initFlag;
    //if have called the resetStroke function
    BOOL m_resetStrokeFlag;
}


- (id)init;
+ (FjRecogEngine*)sharedEngineInstance;
+ (void)releaseShareInstance;
+ (id)allocWithZone:(NSZone *)zone;


- (int)check:(NSString*)appPath;
- (void)initParam;
- (BOOL)initRecogEngine:(NSString*)packagePath;
- (void)endRecogEngine;
- (BOOL)setParam:(int)type m_value:(int*)value m_vLen:(int)vLen;
- (BOOL)clearSavedCandList;
//- (void)addStroke:(int*)x _y:(int*)y _n:(int)n;
- (BOOL)addStroke:(NSMutableArray *)x_Array _y:(NSMutableArray*)y_Array;
- (BOOL)hasPoint;
- (void)resetStroke;
- (const char*)setResult:(int)index;
- (int)getCandIndex:(int)index;
- (BOOL)deleteCand:(int)index;
- (BOOL)replaceCand:(int)h_index _v_index:(int)v_index;
- (const char*)getSavedCandList;
- (BOOL)contextProc;
//===================filter  _USE_EXTERNAL_FILTER =======//
#ifdef _USE_EXTERNAL_FILTER
- (BOOL)setMaxCategory:(int)max;
- (BOOL)setCategory:(int)begin m_end:(int)end m_mode:(int)mode;
- (BOOL)addFilters:(int*)filters m_length:(int)length;
- (BOOL)delFilters:(int*)filters m_length:(int)length;
- (BOOL)setAllFiltered;
- (BOOL)setNonFiltered;
- (BOOL)setCategoryEnd;
#endif
- (int)getLastError;

//===================end filter _USE_EXTERNAL_FILTER =======//
@end
