﻿// Class1.cpp
#include "HWRecogPlugin.h"

#include <stdlib.h> // for wcstombs
#include <Share.h>
#include <stdio.h>

#define MAX_CANDS 31
#define MAX_RESULTS 101

using namespace HWRecogRuntimeComponent;
using namespace Platform;
using namespace Windows::Storage;
using namespace Windows::ApplicationModel;

static FjRecogEngine *g_engine;

bool HWRecogPlugin::InitRecogEngine(Platform::String^ path)
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	if (path == nullptr) {
		return g_engine->initRecogEngine("");
	}

	const wchar_t *W = path->Data();

	char path_c[512];
	wcstombs(path_c, W, 512);

	result = g_engine->initRecogEngine(path_c);

	return result;
}

void HWRecogPlugin::EndRecogEngine()
{
	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	g_engine->endRecogEngine();
}

bool HWRecogPlugin::SetParam(int type, const Platform::Array<int>^ value)
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	switch (type){
	case 0:	//認識域の座標サイズを設定する場合
		if (value == nullptr || value->Length < 2){
			return g_engine->setFrame(-1, -1);
		}
		result = g_engine->setFrame(value[0], value[1]);
		break;
	case 1: //文字認識の返却認識結果数を設定する場合
	case 2: //保存できる最大文字数を設定する場合
	case 3:
		if (value == nullptr || value->Length < 1){
			return g_engine->setParam((FjRecogEngine::SETPARAMTYPE)(type - 1), -1);
		}
		result = g_engine->setParam((FjRecogEngine::SETPARAMTYPE)(type - 1), value[0]);
		break;
	default:
		return false;
	}
	return result;
}

bool HWRecogPlugin::ClearSavedCandList()
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->clearSavedCandList();
	return result;
}

bool HWRecogPlugin::AddStroke(const Platform::Array<Point>^ points)
{
	bool ret = false;
	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	if(points != nullptr){
		int length = points->Length;
		int *x2 = new int[length];
		int *y2 = new int[length];
		for (int i = 0; i < length; i++) {
			x2[i] = points[i].X;
			y2[i] = points[i].Y;
		}
		ret = g_engine->addStroke(x2, y2, length);

		delete[] x2;
		delete[] y2;
	}
	else {
		ret = g_engine->addStroke(nullptr, nullptr, -1);
	}
	return ret;
}

bool HWRecogPlugin::HasPoint()
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->hasPoint();
	return result;
}

void HWRecogPlugin::ResetStroke()
{
	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	g_engine->resetStroke();
}

Platform::String^ HWRecogPlugin::SetResult(int index)
{
	String^ result = nullptr;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	TCHAR cands[MAX_CANDS] = L"";
	int nCands = g_engine->setResult(index, cands);
	if (nCands != -1) {
		Platform::String^ candsText = ref new Platform::String(cands, nCands);
		result = candsText;
	}

	return result;
}

int HWRecogPlugin::GetCandIndex(int index)
{
	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	int idx = g_engine->getCandIndex(index);
	if (idx < 0) {
		return -1;
	}

	return idx;
}

bool HWRecogPlugin::DeleteCand(int index)
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->deleteCand(index);

	return result;
}

bool HWRecogPlugin::ReplaceCand(int h_index, int v_index)
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->replaceCand(h_index, v_index);

	return result;
}

Platform::String^ HWRecogPlugin::GetSavedCandList()
{
	String^ result = nullptr;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	TCHAR str[MAX_RESULTS] = L"";
	bool ret = g_engine->getSavedCandList(str);
	if (ret == true) {
		int strlen = wcslen(str);
		result = ref new Platform::String(str, strlen);
	}
	
	return result;
}

bool HWRecogPlugin::AddFilters(const Platform::Array<int>^ filters)
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	if (filters == nullptr || filters->Length <= 0) {
		return g_engine->addFilters(nullptr, -1);
	}

	int length = filters->Length;
	int *x = new int[length];
	for (int i = 0; i < length; i++) {
		x[i] = filters[i];
	}

	result = g_engine->addFilters(x, length);
	delete[] x;

	return result;
}

bool HWRecogPlugin::SetMaxCategory(int max)
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->setMaxCategory(max);

	return result;
}

bool HWRecogPlugin::SetCategory(unsigned short begin, unsigned short end, int mode)
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->setCategory(begin, end, mode);

	return result;
}

bool HWRecogPlugin::SetCategoryEnd()
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->setCategoryEnd();

	return result;
}

bool HWRecogPlugin::DelFilters(const Platform::Array<int>^ filters)
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	if (filters == nullptr || filters->Length <= 0) {
		return g_engine->delFilters(nullptr, -1);
	}

	int length = filters->Length;
	int *x = new int[length];
	for (int i = 0; i < length; i++) {
		x[i] = filters[i];
	}

	result = g_engine->delFilters(x, length);
	delete[] x;
	return result;
}

bool HWRecogPlugin::SetNonFiltered()
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->setNonFiltered();

	return result;
}

bool HWRecogPlugin::SetAllFiltered()
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->setAllFiltered();

	return result;
}

bool HWRecogPlugin::ContextProc()
{
	bool result;

	if (NULL == g_engine){
		g_engine = FjRecogEngine::createFjRecogEngine();
	}

	result = g_engine->contextProc();

	return result;
}

int HWRecogPlugin::GetLastError()
{
	if (NULL == g_engine) {
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	return g_engine->getLastError();
}

bool HWRecogPlugin::ClearAdaptPatternDic()
{
	if (NULL == g_engine) {
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	return g_engine->clearAdaptPatternDic();
}

int HWRecogPlugin::AdaptPattern(int p_index, int l_index)
{
	if (NULL == g_engine) {
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	return g_engine->adaptPattern(p_index, l_index);
}

Platform::Array<int>^ HWRecogPlugin::GetAdaptPatternIds()
{
	if (NULL == g_engine) {
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	int ids[100];
	Array<int>^ result = nullptr;
	int count = 0;
	if ((count = g_engine->getAdaptPatternIds(ids)) > 0) {
		result = ref new Array<int>(count);
		for (int index = 0; index < count; index++) {
			result[index] = ids[index];
		}
	}
	
	return result;
}

bool HWRecogPlugin::DeleteAdaptPattern(int id)
{
	if (NULL == g_engine) {
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	return g_engine->deleteAdaptPattern(id);
}

wchar_t HWRecogPlugin::GetAdaptPatternChar(int id)
{
	if (NULL == g_engine) {
		g_engine = FjRecogEngine::createFjRecogEngine();
	}
	return (wchar_t)g_engine->getAdaptPatternChar(id);
}